var consumerThumbs = [];
var consumerImages = [];

var canvas = document.getElementById('source');
var sourcectx = canvas.getContext('2d');

var sourceWidth;
var sourceHeight;

var tileX = 0;
var tileY = 0;
var tileWide = 40;
var tileWidth;
var tileHeight;
var tileHigh;

var imageWidth;
var imageHeight;

var tiles = [];
var xYcoords = [];

var grayscale = $('#grayscale');
var consumerImagesDOM;
var mosaicImage = document.getElementById('largeImage');
var start = new Date().getTime();
var handle = new Object();


var isMobile = { 
    Android: function() { return navigator.userAgent.match(/Android/i); }, 
    BlackBerry: function() { return navigator.userAgent.match(/BlackBerry/i); }, 
    iOS: function() { return navigator.userAgent.match(/iPhone|iPad|iPod/i); }, 
    Opera: function() { return navigator.userAgent.match(/Opera Mini/i); }, 
    Windows: function() { return navigator.userAgent.match(/IEMobile/i); }, 
    any: function() { return (isMobile.Android() || isMobile.BlackBerry() || isMobile.iOS() || isMobile.Opera() || isMobile.Windows()); } 
};

var getTheOS = isMobile.any();


function init(callback) {

	var deviceWidth = $(window).width();
	var deviceHeight = $(window).height();

	$(canvas).attr('width', deviceWidth);
	$(canvas).attr('height', deviceHeight);

	sourceWidth = deviceWidth;
	sourceHeight = deviceHeight;

	tileWidth = sourceWidth/tileWide;
	// (original height / original width) x new width = new height
	tileHeight = (768 / 1024) * tileWidth;
	tileHigh = sourceHeight/tileHeight;
	console.log(tileWidth, tileHeight);
	callback();

};



window.requestInterval = function(fn, delay) {
	if( !window.requestAnimationFrame       && 
		!window.webkitRequestAnimationFrame && 
		!(window.mozRequestAnimationFrame && window.mozCancelRequestAnimationFrame) && // Firefox 5 ships without cancel support
		!window.oRequestAnimationFrame      && 
		!window.msRequestAnimationFrame){
			return window.setInterval(fn, delay);
		}
			

		
	function loop() {
		var current = new Date().getTime(),
			delta = current - start;
			
		if(delta >= delay) {
			fn.call();
			start = new Date().getTime();
		}

		handle.value = requestAnimationFrame(loop);
	};
	
	handle.value = requestAnimationFrame(loop);
	return handle;
};


function shuffle(array) {
    var tmp, current, top = array.length;

    if(top) while(--top) {
    	current = Math.floor(Math.random() * (top + 1));
    	tmp = array[current];
    	array[current] = array[top];
    	array[top] = tmp;
    }

    return array;
};


function drawTiles() {
			
	var i;
	var img;
	consumerThumbs = shuffle(consumerThumbs);
	var imageCounter = 0;

		while (tileY < sourceHeight) {

		storeCoordinates(tileX,tileY);

		if (tileX < sourceWidth) {

			if (imageCounter === tiles.length) {imageCounter = 0;}
			// console.log(imageCounter)
			sourcectx.drawImage(tiles[imageCounter], tileX, tileY, tileWidth, tileHeight);
	    	tileX = tileX + tileWidth;
	    	imageCounter++;

		}

		if (tileX >= sourceWidth) {

			tiles = shuffle(tiles);
			tileX = 0;
			tileY = tileY + tileHeight;

		}

	};

	toGrayscale();
	
};


function loadImageArray() {

	var img;
	consumerThumbs = shuffle(consumerThumbs);
	var loadCounter = 0;

	for (var i = 0; i < consumerThumbs.length; i++) {

		var img = new Image(); 
		img.crossOrigin = "anonymous";
		img.src = consumerThumbs[i];	
		tiles.push(img);
		img.onload = function() {

			loadCounter++

			if (loadCounter === consumerThumbs.length) {

				// $.when(drawTiles()).then(toGrayscale()).then(drawFilters());

			 drawTiles(); 

			}
		
		};

	};
};


function getCanvasData() {

	var dataURL = canvas.toDataURL();

	return dataURL;

};



function toGrayscale() {

	var grayscale2 = document.getElementById('grayscale');

	sourcectx.drawImage(grayscale2, 0, 0);

	var imgPixels = sourcectx.getImageData(0, 0, sourceWidth, sourceHeight);


	for(var y = 0; y < imgPixels.height; y++) {
	     for(var x = 0; x < imgPixels.width; x++) {
	          var i = (y * 4) * imgPixels.width + x * 4;
	          var avg = (imgPixels.data[i] + imgPixels.data[i + 1] + imgPixels.data[i + 2]) / 3;
	          imgPixels.data[i] = avg;
	          imgPixels.data[i + 1] = avg;
	          imgPixels.data[i + 2] = avg;
	     }
	}

	sourcectx.putImageData(imgPixels, 0, 0, 0, 0, imgPixels.width, imgPixels.height);

	drawFilters();
	// return canvas.toDataURL();

};


function drawFilters() {

	mosaicImage = new Image(); 
	mosaicImage.crossOrigin = "anonymous";
	mosaicImage.src = document.getElementById('largeImage').src;	
	mosaicImage.onload = function() {

		setTimeout(function() {

			// blendImage('normal');
			// blendImage('color-burn');
			blendImage('overlay');
			blendImage('overlay');
			// blendImage('screen');
			// blendImage('darken');
			// blendImage('multiply');
			// blendImage('lighten');
			// blendImage('hard-light');
			// blendImage('soft-light');

			// blendImage('color-dodge');
			// blendImage('luminosity');
			// blendImage('color');
			// blendImage('hue');
			// blendImage('difference');
			// blendImage('exclusion');

			drawImageTransparent(0.3);

			$('#source').velocity("transition.flipYIn", {
				delay: 1000,
			    complete: function(elements) { 
			    	consumerImagesDOM = document.getElementById('consumerImages').getElementsByTagName('img');
			    	 requestInterval(animateConsumerImages, 5000);
			    }
			});

			$('#loader').velocity("fadeOut", {delay: 500});


		}, 3000);	

	};

};


function blendImage(filter) { 

	sourcectx.globalCompositeOperation = filter;

	setTimeout(function(){ 


			imageWidth = newWidth(mosaicImage);
			imageHeight = sourceHeight;
			
			if (imageWidth > sourceWidth) {

				sourcectx.drawImage(mosaicImage, centerHorizontal(imageWidth), 0, imageWidth, imageHeight);	
			} else {

				imageHeight = newHeight(mosaicImage);

				sourcectx.drawImage(mosaicImage, 0, (sourceHeight - imageHeight)/2, sourceWidth, imageHeight);
			}

	
	 }, 500);

};


function drawImageTransparent(opacity) {

	sourcectx.globalCompositeOperation = 'source-over';
	sourcectx.globalAlpha = opacity;

	setTimeout(function() { 

			imageWidth = newWidth(mosaicImage);
			imageHeight = sourceHeight;

			if (imageWidth > sourceWidth) {

				sourcectx.drawImage(mosaicImage, centerHorizontal(imageWidth), 0, imageWidth, imageHeight);	
			} else {

				imageHeight = newHeight(mosaicImage);

				sourcectx.drawImage(mosaicImage, 0, (sourceHeight - imageHeight)/2, sourceWidth, imageHeight);
			}

	}, 500);

};


function newWidth(img) {

// (original height / original width) x new width = new height
var width = (img.width / img.height) * canvas.height;

return width;

}; 

function newHeight(img) {

// (original height / original width) x new width = new height
var height = (img.height / img.width) * canvas.width;

return height;

};


function storeCoordinates(x,y) {

    xYcoords.push({
        x: x,
        y: y,

    });

};

function selectRandomXy(obj) {

	var rand = obj[Math.floor(Math.random() * obj.length)];

	return rand;

};


function animateConsumerImages() {

		var xy = selectRandomXy(xYcoords);
		animateImage(xy);

};



var imageCounter2 = 0;
var enlargeImage = $('#enlargeImage');
function animateImage(xy) {

	if (xy === undefined) { return }

	var image = consumerImagesDOM[imageCounter2];

	// console.log(xy.x, xy.y);
	$(image).css({"top": xy.y, "left": xy.x });
	// $(image).velocity({ top: "3%", left: "15%", "width": "70%",opacity: 1 }, { duration: 700, easing: "ease-in-out" });

	if (window.matchMedia("(orientation:landscape)").matches) {
  $(image).velocity({ top: "3%", left: "15%", "width": "70%" }, 1625, [500, 20]);
} else {
 $(image).velocity({ top: "20%", left: "5%", "width": "90%" }, 1625, [500, 20]);
}

	

	setTimeout(function() { $(image).velocity("reverse",{duration:500, easing: 'swing'} );}, 3000);

	imageCounter2++;
	if (imageCounter2 === consumerImages.length) { imageCounter2 = 0; };

};


var getConsumerImages = $('#consumerImages');
function appendConsumerImages() {

	getConsumerImages.empty();

	for (var i = consumerImages.length - 1; i >= 0 ; i--) {

		getConsumerImages.append('<img src="' + consumerImages[i] + '">');

	};

};


function redrawMosaic() {console.log('test')

	$('#source').velocity("transition.flipYOut", {delay: 1000});
	$('#loader').velocity("fadeIn");


	var mosaicTimer1 = setTimeout(function() {
		clearTimeout(mosaicTimer1);
		
		imageCounter2 = 0;
		cancelAnimationFrame(handle.value);
		getConsumerImages.empty();
		sourcectx.clearRect(0,0,sourceWidth,sourceHeight);
		appendConsumerImages();
		loadImageArray();
	}, 2000);

};


$('#redraw').on('click', function() {
	
	sourcectx.globalAlpha = 1;
	tileX = 0;
	tileY = 0;
	tiles = [];
	xYcoords = [];
	redrawMosaic();	

});


function startReel() {

	if (counter === 0) {

	loadImageArray();
    appendConsumerImages();

	} else {

		sourcectx.globalAlpha = 1;
		tileX = 0;
		tileY = 0;
		tiles = [];
		xYcoords = [];
		redrawMosaic();	
	}



};


function loadConsumerImageArrays(images) {

	for (var i = 0; i < images.length; i++) {

		consumerThumbs.push(images[i].thumbnailUrl);
		consumerImages.push(images[i].url);
		
	}

};


// returns the x coordinate needed to horizontally center an image. 
function centerHorizontal(imageWidth) {

    var x = canvas.width / 2 - imageWidth / 2;

    return x;

};


// returns the y coordinate needed to vertically center an image. 
function centerVertical(imageHeight) {

    var y = canvas.height / 2 - imageHeight / 2;
    
    return y;

};

var resize = true;
function redrawOnOrientationChange() {

if (getTheOS == 'iPhone'|| getTheOS == 'iPad'|| getTheOS == 'iPod') {

	if ("onorientationchange" in window) {
		// alert('iphone')
		// var supportsOrientationChange = "onorientationchange" in window,
  //   	orientationEvent = supportsOrientationChange ? "orientationchange" : "resize";

	   	window.addEventListener('orientationchange', function() {

	   		if (resize) {

	   			resize = false; 

	   			var resizeTimer = setTimeout(function() {
	   				clearInterval(resizeTimer);
	   				resize = true; 
	   			}, 2000);

			   	var deviceWidth = $(window).width();
				var deviceHeight = $(window).height();

				$(canvas).attr('width', deviceWidth);
				$(canvas).attr('height', deviceHeight);

				sourceWidth = deviceWidth;
				sourceHeight = deviceHeight;

				tileWidth = sourceWidth/tileWide;
				// (original height / original width) x new width = new height
				tileHeight = (768 / 1024) * tileWidth;
				tileHigh = sourceHeight/tileHeight;

			    sourcectx.globalAlpha = 1;
				tileX = 0;
				tileY = 0;
				tiles = [];
				xYcoords = [];
				redrawMosaic();	

	   		}
	   		
		});

	};

} else {
	// alert('android')

	var rtime;
	var timeout = false;
	var delta = 1000;
	$(window).resize(function() {
	    rtime = new Date();
	    if (timeout === false) {
	        timeout = true;
	        setTimeout(resizeend, delta);
	    }
	});

	function resizeend() {
	    if (new Date() - rtime < delta) {
	        setTimeout(resizeend, delta);
	    } else {
	        timeout = false;
	        // alert('Done resizing');
	        			   	var deviceWidth = $(window).width();
					var deviceHeight = $(window).height();

					$(canvas).attr('width', deviceWidth);
					$(canvas).attr('height', deviceHeight);

					sourceWidth = deviceWidth;
					sourceHeight = deviceHeight;

					tileWidth = sourceWidth/tileWide;
					// (original height / original width) x new width = new height
					tileHeight = (768 / 1024) * tileWidth;
					tileHigh = sourceHeight/tileHeight;

				    sourcectx.globalAlpha = 1;
					tileX = 0;
					tileY = 0;
					tiles = [];
					xYcoords = [];
					redrawMosaic();	

			    } 

			}

}



};



$(window).on("load", function() {

    init(Load);
    // redrawOnOrientationChange();

});










